import React, { useState, useEffect } from 'react';
import Chart from 'chart.js/auto';

interface MarketData {
  data_points: {
    date: string;
    values: {
      marketcap: string | number;
      marketcap_change: string | number; 
    };
  }[];
  metrics: {
    marketcap: {
      value: string | number;
      unit: string;
    };
    marketcap_change: {
      value: string | number;
      unit: string;
    };
  };
  data_available_since: string;
}

interface MarketReportProps {
  apiKey: string;
  currency?: string;
  blockchain?: number;
  metrics: string[];
  time_range?: string;
  time_range_start?: Date;
  time_range_end?: Date;
  include_washtrade?: boolean;
}

const MarketTrends: React.FC<MarketReportProps> = ({
  apiKey,
  currency,
  blockchain,
  metrics,
  time_range,
  time_range_start,
  time_range_end,
  include_washtrade,
}) => {
  const [marketData, setMarketData] = useState<MarketData | null>(null);
  const [isLoading, setIsLoading] = useState<boolean>(true);
  const [error, setError] = useState<string | null>(null);
  console.log(marketData)
  useEffect(() => {
    const fetchData = async () => {
      try {
        setIsLoading(true);

        let apiUrl = `https://api.unleashnfts.com/api/v1/market/trend?`;


        if (currency) {
          apiUrl += `currency=${encodeURIComponent(currency)}&`;
        }

    
        if (blockchain) {
          apiUrl += `blockchain=${blockchain}&`;
        }

        
        apiUrl += metrics.map(metric => `metrics=${encodeURIComponent(metric)}`).join('&');

        if (time_range) {
          apiUrl += `&time_range=${encodeURIComponent(time_range)}`;
        }

        if (time_range_start) {
          apiUrl += `&time_range_start=${encodeURIComponent(time_range_start.toISOString())}`;
        }

        if (time_range_end) {
          apiUrl += `&time_range_end=${encodeURIComponent(time_range_end.toISOString())}`;
        }

        if (include_washtrade !== undefined) {
          apiUrl += `&include_washtrade=${include_washtrade}`;
        }

        const response = await fetch(apiUrl, {
          headers: {
            'x-api-key': apiKey,
          },
        });

        if (!response.ok) {
          throw new Error('Failed to fetch data');
        }

        const data: MarketData = await response.json();
        setMarketData(data);
        setError(null);
      } catch (error) {
        setError('Failed to fetch data');
      } finally {
        setIsLoading(false);
      }
    };

    fetchData();
  }, [apiKey, currency, blockchain, metrics, time_range, time_range_start, time_range_end, include_washtrade]);

  useEffect(() => {
    if (!marketData) return;

    const labels: string[] = [];
    const values: (number | null)[] = [];

    for (const point of marketData.data_points) {
      if (point.values.marketcap !== 'NA') {
        labels.push(point.date);
        const marketcap = point.values.marketcap;
        if (typeof marketcap === 'number') {
          values.push(marketcap);
        } else if (typeof marketcap === 'string') {
          const parsedValue = parseFloat(marketcap);
          values.push(isNaN(parsedValue) ? null : parsedValue);
        }
      }
    }

    const ctx = document.getElementById('marketDataChart') as HTMLCanvasElement;
    const chart = new Chart(ctx, {
      type: 'line',
      data: {
        labels: labels,
        datasets: [
          {
            label: 'Marketcap',
            data: values,
            fill: false,
            borderColor: 'rgba(75, 192, 192, 1)',
            tension: 0.1,
          },
        ],
      },
      options: {
        responsive: true,
        plugins: {
          title: {
            display: true,
            text: 'Marketcap Over Time',
          },
        },
        scales: {
          y: {
            beginAtZero: false,
          },
        },
      },
    });

    return () => {
      chart.destroy();
    };
  }, [marketData]);

  if (isLoading) {
    return <div>Loading...</div>;
  }

  if (error) {
    return <div>Error: {error}</div>;
  }

  return <canvas id="marketDataChart" />;
};

export default MarketTrends;
